
/****************************************************************************
 Module : amsultra_bidir_simple - An AMS-Ultra connection module

 VPI task :

$ultra_connect_bidir( Din, Aout, Dreg, vlo, vhi, vthi, vtlo, vx,   
		      vz, tr, tf, tx, tz, txdel,
		      rlo, rhi, rx, rz, vtol, ttol ) 
Din = digital port of connect module
Aout = analog port of connect module
Dreg = driving digital port of connect module

real vlo      // output voltage levels
real vhi      //
real vthi     // upper threshold (def Vhi*2/3)
real vtlo     // lower threshold (def Vhi*1/3)
real vx       // X output voltage
real vz       // Z output voltage
real tr       // risetime (L/X to Hi)
real tf       // falltime (H/X to Lo)
real tx       // time to unknown (L/H/Z to X)
real tz       // impedance risetime to/from Z
real txdel    // analog-midrange to output X
real rlo      // low state output resistance
real rhi      // high state output resistance
real rx       // X state output resistance
real rz       // Z state output resistance 
real vtol     // voltage tolerance
real ttol     // time tolerance of crossing
****************************************************************************/
`include "disciplines.vams"
`timescale 1ns / 100ps

connectmodule amsultra_bidir_simple(Din, Aout);

  inout Din; logic Din;                       // logic signal
  inout Aout; electrical Aout;                // electrical signal


// INSTANCE PARAMETERS:
  parameter real vsup=1.8 from (0:inf);       // nominal supply voltage
  parameter real vlo=0;                       // output voltage levels
  parameter real vhi=vsup from (vlo:vsup];    //
  parameter real vthi=vhi/1.5 from (vlo:vhi); // upper threshold (def Vhi*2/3)
  parameter real vtlo=vthi/2 from (vlo:vthi); // lower threshold (def Vhi*1/3)
  parameter real vx=(vthi+vtlo)/2 from (vtlo:vthi); // X output voltage
  parameter real vz=(vthi+vtlo)/2 from (vlo:vhi);   // Z output voltage
  parameter real tr=0.2n from (0:1m);         // risetime (L/X to Hi)
  parameter real tf=tr from (0:1m);           // falltime (H/X to Lo)
  parameter real tx=tr from (0:1m);           // time to unknown (L/H/Z to X)
  parameter real tz=tr from (0:1m);           // impedance risetime to/from Z
  parameter real txdel=4*tr;                  // analog-midrange to output X
  parameter real rlo=200 from (0:1M);         // low state output resistance
  parameter real rhi=rlo from (0:1M);         // high state output resistance
  parameter real rx=(rlo+rhi)/10 from (0:1M); // X state output resistance
  parameter real rz=10M from (rhi:1T];        // Z state output resistance
  parameter real vtol=vsup/100 from (0:(vthi-vtlo)/4]; // voltage tolerance
  parameter real ttol=tr/4;                   // time tolerance of crossing

  reg Dreg;


  initial
	   $ultra_connect_bidir( Din, Aout, Dreg, vlo, vhi, vthi, vtlo, vx,
                      vz, tr, tf, tx, tz, txdel,
                      rlo, rhi, rx, rz, vtol, ttol );

  assign Din = Dreg;

endmodule
